Package: rsample
Title: General Resampling Infrastructure
Version: 1.3.2
Authors@R: c(
    person("Hannah", "Frick", , "hannah@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6049-5258")),
    person("Fanny", "Chow", , "fannybchow@gmail.com", role = "aut"),
    person("Max", "Kuhn", , "max@posit.co", role = "aut"),
    person("Michael", "Mahoney", , "mike.mahoney.218@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-2402-304X")),
    person("Julia", "Silge", , "julia.silge@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-3671-836X")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Classes and functions to create and summarize different types
    of resampling objects (e.g. bootstrap, cross-validation).
License: MIT + file LICENSE
URL: https://rsample.tidymodels.org,
        https://github.com/tidymodels/rsample
BugReports: https://github.com/tidymodels/rsample/issues
Depends: R (>= 4.1)
Imports: cli, dplyr (>= 1.1.1), furrr, generics, glue, lifecycle,
        methods, pillar, purrr (>= 1.0.0), rlang (>= 1.1.0), slider (>=
        0.1.5), tibble, tidyr, tidyselect, vctrs (>= 0.5.0)
Suggests: broom, covr, ggplot2, knitr, modeldata, recipes (>= 0.1.4),
        rmarkdown, stats, testthat (>= 3.0.0), utils, whisker, withr,
        xml2
VignetteBuilder: knitr
Config/Needs/website: GGally, nlstools, tidymodels,
        tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-23
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-29 18:32:44 UTC; hannah
Author: Hannah Frick [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6049-5258>),
  Fanny Chow [aut],
  Max Kuhn [aut],
  Michael Mahoney [aut] (ORCID: <https://orcid.org/0000-0003-2402-304X>),
  Julia Silge [aut] (ORCID: <https://orcid.org/0000-0002-3671-836X>),
  Hadley Wickham [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hannah Frick <hannah@posit.co>
Repository: CRAN
Date/Publication: 2026-01-30 06:10:15 UTC
