# Generated by roxygen2: do not edit by hand

S3method("[",rset)
S3method("names<-",rset)
S3method(.get_fingerprint,default)
S3method(.get_fingerprint,rset)
S3method(analysis,default)
S3method(analysis,initial_validation_split)
S3method(analysis,rsplit)
S3method(analysis,val_split)
S3method(as.data.frame,rsplit)
S3method(as.integer,rsplit)
S3method(assessment,default)
S3method(assessment,initial_validation_split)
S3method(assessment,internal_calibration_split)
S3method(assessment,rsplit)
S3method(assessment,val_split)
S3method(calibration,default)
S3method(calibration,internal_calibration_split)
S3method(complement,apparent_split)
S3method(complement,default)
S3method(complement,rof_split)
S3method(complement,rsplit)
S3method(complement,sliding_index_split)
S3method(complement,sliding_period_split)
S3method(complement,sliding_window_split)
S3method(dim,initial_validation_split)
S3method(dim,rsplit)
S3method(get_rsplit,default)
S3method(get_rsplit,rset)
S3method(int_bca,bootstraps)
S3method(int_bca,default)
S3method(int_pctl,bootstraps)
S3method(int_pctl,default)
S3method(int_t,bootstraps)
S3method(int_t,default)
S3method(internal_calibration_split,apparent_split)
S3method(internal_calibration_split,boot_split)
S3method(internal_calibration_split,clustering_split)
S3method(internal_calibration_split,default)
S3method(internal_calibration_split,group_boot_split)
S3method(internal_calibration_split,group_initial_validation_split)
S3method(internal_calibration_split,group_mc_split)
S3method(internal_calibration_split,group_val_split)
S3method(internal_calibration_split,group_vfold_split)
S3method(internal_calibration_split,initial_time_split)
S3method(internal_calibration_split,initial_validation_split)
S3method(internal_calibration_split,initial_validation_time_split)
S3method(internal_calibration_split,mc_split)
S3method(internal_calibration_split,sliding_index_split)
S3method(internal_calibration_split,sliding_period_split)
S3method(internal_calibration_split,sliding_window_split)
S3method(internal_calibration_split,time_val_split)
S3method(internal_calibration_split,val_split)
S3method(internal_calibration_split,vfold_split)
S3method(labels,rset)
S3method(labels,rsplit)
S3method(labels,vfold_cv)
S3method(make_splits,data.frame)
S3method(make_splits,default)
S3method(make_splits,list)
S3method(obj_sum,rsplit)
S3method(populate,rset)
S3method(populate,rsplit)
S3method(pretty,apparent)
S3method(pretty,bootstraps)
S3method(pretty,clustering_cv)
S3method(pretty,group_bootstraps)
S3method(pretty,group_mc_cv)
S3method(pretty,group_validation_split)
S3method(pretty,group_vfold_cv)
S3method(pretty,loo_cv)
S3method(pretty,manual_rset)
S3method(pretty,mc_cv)
S3method(pretty,nested_cv)
S3method(pretty,permutations)
S3method(pretty,rolling_origin)
S3method(pretty,sliding_index)
S3method(pretty,sliding_period)
S3method(pretty,sliding_window)
S3method(pretty,validation_set)
S3method(pretty,validation_split)
S3method(pretty,vfold_cv)
S3method(print,apparent)
S3method(print,bootstraps)
S3method(print,clustering_cv)
S3method(print,group_bootstraps)
S3method(print,group_mc_cv)
S3method(print,group_validation_split)
S3method(print,group_vfold_cv)
S3method(print,initial_split)
S3method(print,initial_validation_split)
S3method(print,internal_calibration_split)
S3method(print,loo_cv)
S3method(print,manual_rset)
S3method(print,mc_cv)
S3method(print,nested_cv)
S3method(print,permutations)
S3method(print,rolling_origin)
S3method(print,rsplit)
S3method(print,sliding_index)
S3method(print,sliding_period)
S3method(print,sliding_window)
S3method(print,val_split)
S3method(print,validation_split)
S3method(print,vfold_cv)
S3method(reverse_splits,default)
S3method(reverse_splits,perm_split)
S3method(reverse_splits,permutations)
S3method(reverse_splits,rset)
S3method(reverse_splits,rsplit)
S3method(testing,default)
S3method(testing,initial_validation_split)
S3method(testing,rsplit)
S3method(testing,val_split)
S3method(tidy,nested_cv)
S3method(tidy,rset)
S3method(tidy,rsplit)
S3method(tidy,vfold_cv)
S3method(training,default)
S3method(training,initial_validation_split)
S3method(training,rsplit)
S3method(training,val_split)
S3method(type_sum,rsplit)
S3method(validation,default)
S3method(validation,initial_validation_split)
S3method(validation,val_split)
S3method(vec_cast,apparent.apparent)
S3method(vec_cast,apparent.data.frame)
S3method(vec_cast,apparent.tbl_df)
S3method(vec_cast,bootstraps.bootstraps)
S3method(vec_cast,bootstraps.data.frame)
S3method(vec_cast,bootstraps.tbl_df)
S3method(vec_cast,clustering_cv.clustering_cv)
S3method(vec_cast,clustering_cv.data.frame)
S3method(vec_cast,clustering_cv.tbl_df)
S3method(vec_cast,data.frame.apparent)
S3method(vec_cast,data.frame.bootstraps)
S3method(vec_cast,data.frame.clustering_cv)
S3method(vec_cast,data.frame.group_bootstraps)
S3method(vec_cast,data.frame.group_mc_cv)
S3method(vec_cast,data.frame.group_validation_split)
S3method(vec_cast,data.frame.group_vfold_cv)
S3method(vec_cast,data.frame.loo_cv)
S3method(vec_cast,data.frame.manual_rset)
S3method(vec_cast,data.frame.mc_cv)
S3method(vec_cast,data.frame.nested_cv)
S3method(vec_cast,data.frame.permutations)
S3method(vec_cast,data.frame.rolling_origin)
S3method(vec_cast,data.frame.sliding_index)
S3method(vec_cast,data.frame.sliding_period)
S3method(vec_cast,data.frame.sliding_window)
S3method(vec_cast,data.frame.validation_set)
S3method(vec_cast,data.frame.validation_split)
S3method(vec_cast,data.frame.validation_time_split)
S3method(vec_cast,data.frame.vfold_cv)
S3method(vec_cast,group_bootstraps.data.frame)
S3method(vec_cast,group_bootstraps.group_bootstraps)
S3method(vec_cast,group_bootstraps.tbl_df)
S3method(vec_cast,group_mc_cv.data.frame)
S3method(vec_cast,group_mc_cv.group_mc_cv)
S3method(vec_cast,group_mc_cv.tbl_df)
S3method(vec_cast,group_validation_split.data.frame)
S3method(vec_cast,group_validation_split.group_validation_split)
S3method(vec_cast,group_validation_split.tbl_df)
S3method(vec_cast,group_vfold_cv.data.frame)
S3method(vec_cast,group_vfold_cv.group_vfold_cv)
S3method(vec_cast,group_vfold_cv.tbl_df)
S3method(vec_cast,loo_cv.data.frame)
S3method(vec_cast,loo_cv.loo_cv)
S3method(vec_cast,loo_cv.tbl_df)
S3method(vec_cast,manual_rset.data.frame)
S3method(vec_cast,manual_rset.manual_rset)
S3method(vec_cast,manual_rset.tbl_df)
S3method(vec_cast,mc_cv.data.frame)
S3method(vec_cast,mc_cv.mc_cv)
S3method(vec_cast,mc_cv.tbl_df)
S3method(vec_cast,nested_cv.data.frame)
S3method(vec_cast,nested_cv.nested_cv)
S3method(vec_cast,nested_cv.tbl_df)
S3method(vec_cast,permutations.data.frame)
S3method(vec_cast,permutations.permutations)
S3method(vec_cast,permutations.tbl_df)
S3method(vec_cast,rolling_origin.data.frame)
S3method(vec_cast,rolling_origin.rolling_origin)
S3method(vec_cast,rolling_origin.tbl_df)
S3method(vec_cast,sliding_index.data.frame)
S3method(vec_cast,sliding_index.sliding_index)
S3method(vec_cast,sliding_index.tbl_df)
S3method(vec_cast,sliding_period.data.frame)
S3method(vec_cast,sliding_period.sliding_period)
S3method(vec_cast,sliding_period.tbl_df)
S3method(vec_cast,sliding_window.data.frame)
S3method(vec_cast,sliding_window.sliding_window)
S3method(vec_cast,sliding_window.tbl_df)
S3method(vec_cast,tbl_df.apparent)
S3method(vec_cast,tbl_df.bootstraps)
S3method(vec_cast,tbl_df.clustering_cv)
S3method(vec_cast,tbl_df.group_bootstraps)
S3method(vec_cast,tbl_df.group_mc_cv)
S3method(vec_cast,tbl_df.group_validation_split)
S3method(vec_cast,tbl_df.group_vfold_cv)
S3method(vec_cast,tbl_df.loo_cv)
S3method(vec_cast,tbl_df.manual_rset)
S3method(vec_cast,tbl_df.mc_cv)
S3method(vec_cast,tbl_df.nested_cv)
S3method(vec_cast,tbl_df.permutations)
S3method(vec_cast,tbl_df.rolling_origin)
S3method(vec_cast,tbl_df.sliding_index)
S3method(vec_cast,tbl_df.sliding_period)
S3method(vec_cast,tbl_df.sliding_window)
S3method(vec_cast,tbl_df.validation_set)
S3method(vec_cast,tbl_df.validation_split)
S3method(vec_cast,tbl_df.validation_time_split)
S3method(vec_cast,tbl_df.vfold_cv)
S3method(vec_cast,validation_set.data.frame)
S3method(vec_cast,validation_set.tbl_df)
S3method(vec_cast,validation_set.validation_set)
S3method(vec_cast,validation_split.data.frame)
S3method(vec_cast,validation_split.tbl_df)
S3method(vec_cast,validation_split.validation_split)
S3method(vec_cast,validation_time_split.data.frame)
S3method(vec_cast,validation_time_split.tbl_df)
S3method(vec_cast,validation_time_split.validation_time_split)
S3method(vec_cast,vfold_cv.data.frame)
S3method(vec_cast,vfold_cv.tbl_df)
S3method(vec_cast,vfold_cv.vfold_cv)
S3method(vec_ptype2,apparent.apparent)
S3method(vec_ptype2,apparent.data.frame)
S3method(vec_ptype2,apparent.tbl_df)
S3method(vec_ptype2,bootstraps.bootstraps)
S3method(vec_ptype2,bootstraps.data.frame)
S3method(vec_ptype2,bootstraps.tbl_df)
S3method(vec_ptype2,clustering_cv.clustering_cv)
S3method(vec_ptype2,clustering_cv.data.frame)
S3method(vec_ptype2,clustering_cv.tbl_df)
S3method(vec_ptype2,data.frame.apparent)
S3method(vec_ptype2,data.frame.bootstraps)
S3method(vec_ptype2,data.frame.clustering_cv)
S3method(vec_ptype2,data.frame.group_bootstraps)
S3method(vec_ptype2,data.frame.group_mc_cv)
S3method(vec_ptype2,data.frame.group_validation_split)
S3method(vec_ptype2,data.frame.group_vfold_cv)
S3method(vec_ptype2,data.frame.loo_cv)
S3method(vec_ptype2,data.frame.manual_rset)
S3method(vec_ptype2,data.frame.mc_cv)
S3method(vec_ptype2,data.frame.nested_cv)
S3method(vec_ptype2,data.frame.permutations)
S3method(vec_ptype2,data.frame.rolling_origin)
S3method(vec_ptype2,data.frame.sliding_index)
S3method(vec_ptype2,data.frame.sliding_period)
S3method(vec_ptype2,data.frame.sliding_window)
S3method(vec_ptype2,data.frame.validation_set)
S3method(vec_ptype2,data.frame.validation_split)
S3method(vec_ptype2,data.frame.validation_time_split)
S3method(vec_ptype2,data.frame.vfold_cv)
S3method(vec_ptype2,group_bootstraps.data.frame)
S3method(vec_ptype2,group_bootstraps.group_bootstraps)
S3method(vec_ptype2,group_bootstraps.tbl_df)
S3method(vec_ptype2,group_mc_cv.data.frame)
S3method(vec_ptype2,group_mc_cv.group_mc_cv)
S3method(vec_ptype2,group_mc_cv.tbl_df)
S3method(vec_ptype2,group_validation_split.data.frame)
S3method(vec_ptype2,group_validation_split.group_validation_split)
S3method(vec_ptype2,group_validation_split.tbl_df)
S3method(vec_ptype2,group_vfold_cv.data.frame)
S3method(vec_ptype2,group_vfold_cv.group_vfold_cv)
S3method(vec_ptype2,group_vfold_cv.tbl_df)
S3method(vec_ptype2,loo_cv.data.frame)
S3method(vec_ptype2,loo_cv.loo_cv)
S3method(vec_ptype2,loo_cv.tbl_df)
S3method(vec_ptype2,manual_rset.data.frame)
S3method(vec_ptype2,manual_rset.manual_rset)
S3method(vec_ptype2,manual_rset.tbl_df)
S3method(vec_ptype2,mc_cv.data.frame)
S3method(vec_ptype2,mc_cv.mc_cv)
S3method(vec_ptype2,mc_cv.tbl_df)
S3method(vec_ptype2,nested_cv.data.frame)
S3method(vec_ptype2,nested_cv.nested_cv)
S3method(vec_ptype2,nested_cv.tbl_df)
S3method(vec_ptype2,permutations.data.frame)
S3method(vec_ptype2,permutations.permutations)
S3method(vec_ptype2,permutations.tbl_df)
S3method(vec_ptype2,rolling_origin.data.frame)
S3method(vec_ptype2,rolling_origin.rolling_origin)
S3method(vec_ptype2,rolling_origin.tbl_df)
S3method(vec_ptype2,sliding_index.data.frame)
S3method(vec_ptype2,sliding_index.sliding_index)
S3method(vec_ptype2,sliding_index.tbl_df)
S3method(vec_ptype2,sliding_period.data.frame)
S3method(vec_ptype2,sliding_period.sliding_period)
S3method(vec_ptype2,sliding_period.tbl_df)
S3method(vec_ptype2,sliding_window.data.frame)
S3method(vec_ptype2,sliding_window.sliding_window)
S3method(vec_ptype2,sliding_window.tbl_df)
S3method(vec_ptype2,tbl_df.apparent)
S3method(vec_ptype2,tbl_df.bootstraps)
S3method(vec_ptype2,tbl_df.clustering_cv)
S3method(vec_ptype2,tbl_df.group_bootstraps)
S3method(vec_ptype2,tbl_df.group_mc_cv)
S3method(vec_ptype2,tbl_df.group_validation_split)
S3method(vec_ptype2,tbl_df.group_vfold_cv)
S3method(vec_ptype2,tbl_df.loo_cv)
S3method(vec_ptype2,tbl_df.manual_rset)
S3method(vec_ptype2,tbl_df.mc_cv)
S3method(vec_ptype2,tbl_df.nested_cv)
S3method(vec_ptype2,tbl_df.permutations)
S3method(vec_ptype2,tbl_df.rolling_origin)
S3method(vec_ptype2,tbl_df.sliding_index)
S3method(vec_ptype2,tbl_df.sliding_period)
S3method(vec_ptype2,tbl_df.sliding_window)
S3method(vec_ptype2,tbl_df.validation_set)
S3method(vec_ptype2,tbl_df.validation_split)
S3method(vec_ptype2,tbl_df.validation_time_split)
S3method(vec_ptype2,tbl_df.vfold_cv)
S3method(vec_ptype2,validation_set.data.frame)
S3method(vec_ptype2,validation_set.tbl_df)
S3method(vec_ptype2,validation_set.validation_set)
S3method(vec_ptype2,validation_split.data.frame)
S3method(vec_ptype2,validation_split.tbl_df)
S3method(vec_ptype2,validation_split.validation_split)
S3method(vec_ptype2,validation_time_split.data.frame)
S3method(vec_ptype2,validation_time_split.tbl_df)
S3method(vec_ptype2,validation_time_split.validation_time_split)
S3method(vec_ptype2,vfold_cv.data.frame)
S3method(vec_ptype2,vfold_cv.tbl_df)
S3method(vec_ptype2,vfold_cv.vfold_cv)
S3method(vec_ptype_abbr,apparent)
S3method(vec_ptype_abbr,bootstraps)
S3method(vec_ptype_abbr,clustering_cv)
S3method(vec_ptype_abbr,group_bootstraps)
S3method(vec_ptype_abbr,group_initial_split)
S3method(vec_ptype_abbr,group_initial_validation_split)
S3method(vec_ptype_abbr,group_mc_cv)
S3method(vec_ptype_abbr,group_validation_split)
S3method(vec_ptype_abbr,group_vfold_cv)
S3method(vec_ptype_abbr,initial_split)
S3method(vec_ptype_abbr,initial_time_split)
S3method(vec_ptype_abbr,initial_validation_split)
S3method(vec_ptype_abbr,manual_rset)
S3method(vec_ptype_abbr,nested_cv)
S3method(vec_ptype_abbr,permutations)
S3method(vec_ptype_abbr,rolling_origin)
S3method(vec_ptype_abbr,sliding_index)
S3method(vec_ptype_abbr,sliding_period)
S3method(vec_ptype_abbr,sliding_window)
S3method(vec_ptype_abbr,validation_set)
S3method(vec_ptype_abbr,validation_split)
S3method(vec_ptype_abbr,validation_time_split)
S3method(vec_ptype_abbr,vfold_cv)
S3method(vec_restore,apparent)
S3method(vec_restore,bootstraps)
S3method(vec_restore,clustering_cv)
S3method(vec_restore,group_bootstraps)
S3method(vec_restore,group_mc_cv)
S3method(vec_restore,group_validation_split)
S3method(vec_restore,group_vfold_cv)
S3method(vec_restore,loo_cv)
S3method(vec_restore,manual_rset)
S3method(vec_restore,mc_cv)
S3method(vec_restore,nested_cv)
S3method(vec_restore,permutations)
S3method(vec_restore,rolling_origin)
S3method(vec_restore,sliding_index)
S3method(vec_restore,sliding_period)
S3method(vec_restore,sliding_window)
S3method(vec_restore,validation_set)
S3method(vec_restore,validation_split)
S3method(vec_restore,validation_time_split)
S3method(vec_restore,vfold_cv)
export(.get_fingerprint)
export(.get_split_args)
export(add_resample_id)
export(all_of)
export(analysis)
export(any_of)
export(apparent)
export(assessment)
export(bootstraps)
export(calibration)
export(caret2rsample)
export(clustering_cv)
export(complement)
export(contains)
export(ends_with)
export(everything)
export(form_pred)
export(get_rsplit)
export(group_bootstraps)
export(group_initial_split)
export(group_initial_validation_split)
export(group_mc_cv)
export(group_validation_split)
export(group_vfold_cv)
export(initial_split)
export(initial_time_split)
export(initial_validation_split)
export(initial_validation_time_split)
export(int_bca)
export(int_pctl)
export(int_t)
export(internal_calibration_split)
export(last_col)
export(loo_cv)
export(make_splits)
export(make_strata)
export(manual_rset)
export(matches)
export(mc_cv)
export(nested_cv)
export(new_rset)
export(num_range)
export(permutations)
export(populate)
export(reg_intervals)
export(reshuffle_rset)
export(reverse_splits)
export(rolling_origin)
export(rsample2caret)
export(rset_reconstruct)
export(sliding_index)
export(sliding_period)
export(sliding_window)
export(starts_with)
export(testing)
export(tidy)
export(training)
export(validation)
export(validation_set)
export(validation_split)
export(validation_time_split)
export(vfold_cv)
import(rlang)
import(vctrs)
importFrom(cli,cli_abort)
importFrom(cli,cli_text)
importFrom(cli,cli_warn)
importFrom(dplyr,arrange)
importFrom(dplyr,arrange_)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,do)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,last)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(furrr,future_map)
importFrom(generics,tidy)
importFrom(lifecycle,deprecated)
importFrom(methods,formalArgs)
importFrom(pillar,type_sum)
importFrom(purrr,list_rbind)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_dbl)
importFrom(purrr,map_lgl)
importFrom(purrr,pluck)
importFrom(stats,dist)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tibble,obj_sum)
importFrom(tibble,tibble)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,any_of)
importFrom(tidyselect,contains)
importFrom(tidyselect,ends_with)
importFrom(tidyselect,everything)
importFrom(tidyselect,last_col)
importFrom(tidyselect,matches)
importFrom(tidyselect,num_range)
importFrom(tidyselect,one_of)
importFrom(tidyselect,starts_with)
importFrom(tidyselect,vars_select)
importFrom(utils,globalVariables)
importFrom(vctrs,vec_count)
importFrom(vctrs,vec_in)
importFrom(vctrs,vec_ptype_abbr)
importFrom(vctrs,vec_slice)
importFrom(vctrs,vec_unique_count)
