#' Job Attrition
#'
#' @details These data are from the IBM Watson Analytics Lab.
#'  The website describes the data with \dQuote{Uncover the
#'  factors that lead to employee attrition and explore important
#'  questions such as \sQuote{show me a breakdown of distance
#'  from home by job role and attrition} or \sQuote{compare
#'  average monthly income by education and attrition}. This is a
#'  fictional data set created by IBM data scientists.}. There
#'  are 1470 rows.
#'
#' These data are now in the `modeldata` package.
#'
#' @name attrition
NULL


#' Two Class Data
#'
#' @details There are artificial data with two predictors (`A` and `B`) and
#'  a factor outcome variable (`Class`).
#'
#' These data are now in the `modeldata` package.
#'
#' @name two_class_dat
NULL

#' Sample Time Series Data
#'
#' @details Drink sales. The exact name of the series from FRED is:
#' "Merchant Wholesalers, Except Manufacturers' Sales Branches and Offices
#' Sales: Nondurable Goods: Beer, Wine, and Distilled Alcoholic Beverages Sales"
#'
#' These data are now in the `modeldata` package.
#' @name drinks
NULL
