---
title: "Time Series Analysis Example"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Time Series Analysis Example}
output:
  knitr:::html_vignette:
    toc: yes
---

This vignette has been moved to [`tidymodels.org`](https://www.tidymodels.org/learn/models/time-series/). 
