# accessor functions for `val_split`

    Code
      testing(val_split)
    Condition
      Error in `testing()`:
      ! The testing data is not part of the validation set object.
      i It is part of the result of the initial 3-way split, e.g., with `initial_validation_split()`.

