---
title: "Grid Search Tuning of Keras Models"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Grid Search Tuning of Keras Models}
output:
  knitr:::html_vignette:
    toc: yes
---

Since `keras` models can be tuning using the `tune` package, this vignette is deprecated. The last version of it can be found [here](https://github.com/tidymodels/rsample/blob/8333b7bb441e0097418205dd735bd3f0cc418916/vignettes/Applications/Keras.Rmd). 
